/*
 * naalaa_image.h
 * --------------
 *
 * By: Marcus 2021
 */
 

#ifndef __PL_IMAGE_H__
#define __PL_IMAGE_H__

typedef struct {
    char hasAlpha;
    /*char hasColorKey;*/
} ImageCellInfo;

/* Image. */
typedef struct {
    int id;
    unsigned int *buffer;
    ImageCellInfo *cellInfo;
    char hasAlpha;
    char hasColorKey;
    unsigned int colorKey;
    int w;
    int h;
    int cols;
    int rows;
    int cells;
    int xMin;
    int yMin;
    int xMax;
    int yMax;
} Image;

/* Macros for color components. */

#define ToRGB(r, g, b) \
    (unsigned int)(b + ((g) << 8) + ((r) << 16) + (128 << 24))

#define ToRGBA(r, g, b, a) \
    (unsigned int)(b + ((g) << 8) + ((r) << 16) + ((a) << 24))

#define ColorRedComponent(c) \
    (unsigned char)((c) >> 16)

#define ColorGreenComponent(c) \
    (unsigned char)((c) >> 8)

#define ColorBlueComponent(c) \
    (unsigned char)(c)

#define ColorAlphaComponent(c) \
    (unsigned char)((c) >> 24)

#define ColorToRGBComponents(c, r, g, b) \
    (r) = (c) >> 16;\
    (g) = (c) >> 8;\
    (b) = (c)
    
#define ColorToRGBAComponents(c, r, g, b, a) \
    (r) = (c) >> 16;\
    (g) = (c) >> 8;\
    (b) = (c);\
    (a) = (c) >> 24

/*
 * IMG_Create
 * ----------
 */
Image *IMG_Create(int w, int h, unsigned int color);

/*
 * IMG_FromBuffer
 * --------------
 */
Image *IMG_FromBuffer(unsigned int *buffer, int w, int h);

/*
 * IMG_Load
 * --------
 */
Image *IMG_Load(const char *filename);

/*
 * IMG_Save
 * --------
 */
int IMG_Save(Image *img, const char *filename);

/*
 * IMG_Free
 * --------
 */
void IMG_Free(Image *img);


/*
 * IMG_BufferChanged
 * -----------------
 */
void IMG_BufferChanged(Image *img);

/*
 * IMG_SetColorKey
 * ---------------
 */
void IMG_SetColorKey(Image *img, unsigned int color);

/*
 * IMG_SetGrid
 * -----------
 */
void IMG_SetGrid(Image *img, int cols, int rows);

/*
 * IMG_Width
 * ---------
 */
int IMG_Width(Image *img);

/*
 * IMG_Height
 * ----------
 */
int IMG_Height(Image *img);

/*
 * IMG_Buffer
 * ----------
 */
unsigned int *IMG_Buffer(Image *img);

/*
 * IMG_SetClipRect
 * ---------------
 */
void IMG_SetClipRect(Image *img, int x, int y, int w, int h);

/*
 * IMG_ClipRect
 * ------------
 */
void IMG_ClipRect(Image *img, int x, int y, int w, int h);

/*
 * IMG_ClearClipRect
 * -----------------
 */
void IMG_ClearClipRect(Image *img);

/*
 * IMG_ClipX
 * ---------
 */
int IMG_ClipX(Image *img);

/*
 * IMG_ClipY
 * ---------
 */
int IMG_ClipY(Image *img);

/*
 * IMG_ClipWidth
 * -------------
 */
int IMG_ClipWidth(Image *img);

/*
 * IMG_ClipHeight
 * --------------
 */
int IMG_ClipHeight(Image *img);

/*
 * IMG_DrawImage
 * -------------
 */
void IMG_DrawImage(Image *dst, int x, int y, Image *src, int srcX, int srcY, int srcW, int srcH, unsigned int color, int useImageAlpha);

/*
 * IMG_DrawImageCel
 * ----------------
 */
void IMG_DrawImageCel(Image *dst, int x, int y, Image *src, int cel, unsigned int color);

/*
 * IMG_DrawVRaster
 * ---------------
 */
void IMG_DrawVRaster(Image *dst, Image *src,
        int x, int y0, int y1,
        float srcU, float srcV, float dstU, float dstV, unsigned int color);

/*
 * IMG_DrawHRaster
 * ---------------
 */
void IMG_DrawHRaster(Image *dst, Image *src,
        int y, int x0, int x1,
        float srcU, float srcV, float dstU, float dstV, unsigned int color);

/*
 * IMG_DrawPixel
 * -------------
 */
void IMG_DrawPixel(Image *dst, int x, int y, unsigned int color);

/*
 * IMG_SetPixel
 * ------------
 */
void IMG_SetPixel(Image *dst, int x, int y, unsigned int color);

/*
 * IMG_GetPixel
 * ------------
 */
int IMG_GetPixel(Image *img, int x, int y, unsigned int *color);

/*
 * IMG_DrawRect
 * ------------
 */
void IMG_DrawRect(Image *dst, int x, int y, int w, int h, unsigned int color);

/*
 * IMG_FillRect
 * ------------
 */
void IMG_FillRect(Image *dst, int x, int y, int w, int h, unsigned int color);

/*
 * IMG_ClearRect
 * -------------
 */
void IMG_ClearRect(Image *dst, int x, int y, int w, int h);

/*
 * IMG_DrawLine
 * ------------
 */
void IMG_DrawLine(Image *dst, int x0, int y0, int x1, int y1, unsigned int color);

/*
 * IMG_DrawEllipse
 * ---------------
 */
void IMG_DrawEllipse(Image *dst, int cx, int cy, int xr, int yr, unsigned int color);

/*
 * IMG_FillEllipse
 * ---------------
 */
void IMG_FillEllipse(Image *dst, int cx, int cy, int xr, int yr, unsigned int color);

/*
 * IMG_DrawPolygon
 * ---------------
 */
void IMG_DrawPolygon(Image *dst, int numPoints, int *points, unsigned int color);

/*
 * IMG_DrawPolygon
 * ---------------
 */
void IMG_FillPolygon(Image *dst, int numPoints, int *points, unsigned int color);

#endif
